@@ -7,7 +7,7 @@ from vxbase import logger
 
 from vxci import __version__, common
 from vxci.checkers import check_all
-from vxci.common import check_credentials, prepare, resume_log, notify_orchest
+from vxci.common import check_credentials, prepare, resume_log, notify_orchest, generate_coverage_reports
 from vxci.coverage import push_coverage_result
 from vxci.image import push_image
 import signal
@@ -109,7 +109,9 @@ def build_image(**kwargs):
         common.push_image(config, config['instance_image'], config['image_tag'])
         common.push_image(config, config['instance_image'], config['image_git_tag'])
         common.notify_orchest(config, is_latest=is_latest)
-        common.save_imagename(config)
+        common.save_imagename(config["ci_commit_ref_slug"], config)
+        # TODO: Old compatibility, remove this line after all repo are migrated
+        common.save_imagename(config["ci_commit_ref_name"], config)
     common.clear_images(config)
     sys.exit(0)
 
@@ -298,9 +300,60 @@ def test_repo(**kwargs):
 
 @cli.command()
 def push_coverage():
+    _logger.warning("Deprecated command ' - vxci push_coverage'. "
+                    "Use\n - source variables.sh\n - vxci push_coverage_combine")
     push_coverage_result()
 
 
+@cli.command()
+@click.option('--ci_commit_ref_name', default=environ.get('CI_COMMIT_REF_NAME'),
+              help=("The branch or tag name for which project is built."
+                    " Env var: CI_COMMIT_REF_NAME."))
+@click.option('--ci_pipeline_id', default=environ.get('CI_PIPELINE_ID'),
+              help=("The unique id of the current pipeline that GitLab CI"
+                    " uses internally. Env var: CI_PIPELINE_ID."))
+@click.option('--ci_repository_url', default=environ.get('CI_REPOSITORY_URL'),
+              help=("The URL to clone the Git repository."
+                    " Env var: CI_REPOSITORY_URL."))
+@click.option('--base_image', default=environ.get('BASE_IMAGE'),
+              help="Env var: BASE_IMAGE.")
+@click.option('--odoo_repo', default=environ.get('ODOO_REPO'),
+              help=("Env var: ODOO_REPO."))
+@click.option('--odoo_branch', default=environ.get('ODOO_BRANCH'),
+              help=("Env var: ODOO_BRANCH."))
+@click.option('--version', default=environ.get('VERSION'),
+              help=("Env var: VERSION."))
+@click.option('--install', default=environ.get('MAIN_APP'),
+              help=("Env var: MAIN_APP."))
+@click.option('--ci_job_id', default=environ.get('CI_JOB_ID'),
+              help=("The unique id of the current job that GitLab CI uses internally."
+                    " Env var: CI_JOB_ID."))
+@click.option('--psql_image', default=False,
+              help=("Override the default postgresql image to use for the tests"
+                    "(Notice that this will override the PSQL_VERSION too)"))
+@click.option('--image_repo_url', default=environ.get('IMAGE_REPO_URL', "quay.io/vauxoo"),
+              help=("The URL where the image repository is located."
+                    " Env var: IMAGE_REPO_URL."))
+@click.option('--push_image', is_flag=True,
+              help="If set it will push the image when on the main branch after the tests")
+@click.option('--git_commit_sha', default=environ.get('CI_COMMIT_SHA'),
+              help=("'git rev-parse HEAD' of the main repository."
+                    "Env var: CI_COMMIT_SHA."))
+def push_coverage_combine(**kwargs):
+    config = common.prepare(**kwargs)
+    status, msg = generate_coverage_reports(config)
+
+    if status:
+        green_pattern = "\033[1;32m\033[1;49m%s\033[0m"
+        _logger.info(green_pattern, "Coverage succeeded!")
+    else:
+        red_pattern = "\033[1;31m\033[1;49m%s\033[0m"
+        _logger.error(red_pattern, msg)
+
+    push_coverage_result()
+    exit(not status)
+
+
 @cli.command()
 @click.option('--ci_project_name', default=environ.get('CI_PROJECT_NAME'),
               help=("The project name that is currently being built."
