@@ -370,27 +370,32 @@ def prepare(**kwargs):
     return config
 
 
-def save_imagename(config):
-    if path.isfile(config["ci_commit_ref_name"]):
-        remove(config["ci_commit_ref_name"])
-    if not path.isdir(config["ci_commit_ref_name"]):
-        makedirs(config["ci_commit_ref_name"])
-    filename = path.join(config["ci_commit_ref_name"], "image_name.env")
+def save_imagename(work_dir, config):
+    if path.isfile(work_dir):
+        remove(work_dir)
+    if not path.isdir(work_dir):
+        makedirs(work_dir)
+    filename = path.join(work_dir, "image_name.env")
     with open(filename, "w") as f:
         img = config.get('docker_image_repo') or config['image_repo']
         f.write("export IMAGE_REPO={}\n".format(img))
         f.write("export IMAGE_TAG={}\n".format(config['image_tag']))
 
 
-def start_instance(config):
+def get_coverage_env_variables(config):
     coverage_info = {
         "COVERAGE_RCFILE": "%(coverage_reports)s/.coveragerc" % config,
-        "COVERAGE_FILE": "%(coverage_reports)s/.coverage" % config,
+        "COVERAGE_FILE": "%(coverage_reports)s/.coverage_%(ci_job_name)s" % config,
         "COVERAGE_HOME": "/home/odoo/instance/extra_addons/%(ci_project_name)s" % config,
         "COVERAGE_CONTEXT": "job_%(ci_job_name)s" % config,
         "EXCLUDE_COVERAGE": config.get("exclude_coverage"),
         "COVERAGE_PRECISION": config.get("coverage_precision"),
     }
+    return coverage_info
+
+
+def start_instance(config):
+    coverage_info = get_coverage_env_variables(config)
     env = {
         "DB_USER": "odoo",
         "DB_PASSWORD": "odoo",
@@ -533,6 +538,32 @@ def install_module(config):
     return True
 
 
+def container_coverage_reports(config, combine=False):
+    cover_reports = []
+    if combine:
+        # Use /bin/sh to propagate the "_*" into the container instead of the host
+        cmd = '/bin/sh -c "coverage combine --keep {coverage_reports}/.coverage_*"'.format(**config)
+        cover_reports.append(cmd)
+
+    cover_reports.extend([
+        'coverage html -d {coverage_reports}/htmlcov'.format(**config),
+        'coverage xml -o {coverage_reports}/coverage.xml'.format(**config),
+        'coverage report -m --show-missing',
+    ])
+
+    status, msg = (True, "")
+    for cr in cover_reports:
+        cmd_res = exec_cmd(config['container_name'], cr, 'odoo', stream=False, workdir=config.get('coverage_home'))
+        _logger.info("Coverage reports command: %s Result:\n%s", cr, cmd_res)
+        if status and "Coverage failure" in cmd_res:
+            status, msg = (False, cmd_res.splitlines()[-1])
+
+    cmd = 'docker cp {container_name}:{coverage_reports}/. {container_vol}/.'.format(**config)
+    subprocess.call(shlex.split(cmd))
+
+    return (status, msg)
+
+
 def test_module(config):
     modules2test = get_modules2test('.', config.get('include'), config.get('exclude'))
     odoo_test_tags = set(i.strip() for i in (config.get('odoo_test_tags') or '').strip().split(',')) - {''}
@@ -554,6 +585,7 @@ def test_module(config):
     _logger.info('Stopping odoo')
     exec_cmd(config['container_name'], 'supervisorctl stop odoo')
     prepare_coverage(config)
+    install_test_deps(config)
 
     _logger.info('\nRunning tests for %s', modules2test)
     _logger.info('Command : %s', test_command)
@@ -563,22 +595,7 @@ def test_module(config):
     show_log(test_log[1], 'Installation with demo and tests', config)
     save_logs(config, test_res)
 
-    # subprocess.call(['docker', 'cp', config.get('container_name')+':/tmp/overage', cover_data])
-
-    cover_reports = [
-        'coverage report -m --show-missing',
-        'coverage html -d {coverage_reports}/htmlcov'.format(**config),
-        'coverage xml -o {coverage_reports}/coverage.xml'.format(**config),
-    ]
-
-    for cr in cover_reports:
-        _logger.info('Running command: %s', cr)
-        cmd_res = exec_cmd(config['container_name'], cr, 'odoo', stream=False, workdir=config.get('coverage_home'))
-        _logger.info('Command: "%s" exited with:\n%s', cr, cmd_res)
-
-    cmd = 'docker cp {container_name}:{coverage_reports}/. {container_vol}/.'.format(**config)
-    subprocess.call(shlex.split(cmd))
-
+    container_coverage_reports(config)
 
     return test_log[0]
 
@@ -589,23 +606,8 @@ def copy_files(src, dst):
         if path.isfile(source):
             copy(source, destination)
 
-def prepare_coverage(config):
-    if not path.isdir(config.get('container_vol')):
-        makedirs(config.get('container_vol'))
-    fcoveragerc = path.join(config.get('container_vol'), '.coveragerc')
-    with open(fcoveragerc, "w") as f_coveragerc:
-        f_coveragerc.write(get_coverage_content(config))
-
-    _logger.debug('Create workdir %s', exec_cmd(config['container_name'], 'mkdir -p {coverage_reports}'.format(**config), user='odoo'))
-
-    cp_cover_cmd = 'docker cp {coverrc} {container}:{folder}'.format(
-        coverrc=fcoveragerc,
-        container=config.get('container_name'),
-        folder=config.get('coverage_rcfile'))
-    subprocess.call(shlex.split(cp_cover_cmd))
-
-    _logger.debug('Changed permissions for coverage %s', exec_cmd(config['container_name'], 'chown odoo:odoo {coverage_rcfile}'.format(**config)))
 
+def install_test_deps(config):
     _logger.debug('Installing dependencies to run tests')
     exec_cmd(config.get('container_name'), 'touch /home/odoo/full_test-requirements.txt')
     exec_cmd(config.get('container_name'), 'pip3 install -r /home/odoo/full_test-requirements.txt')
@@ -617,6 +619,23 @@ def prepare_coverage(config):
         exec_cmd(config.get('container_name'), install_chrome_command)
 
 
+def prepare_coverage(config):
+    if not path.isdir(config.get('container_vol')):
+        makedirs(config.get('container_vol'))
+    fcoveragerc = path.join(config.get('container_vol'), '.coveragerc')
+    with open(fcoveragerc, "w") as f_coveragerc:
+        f_coveragerc.write(get_coverage_content(config))
+
+    _logger.debug('Create workdir %s', exec_cmd(config['container_name'], 'mkdir -p {coverage_reports}'.format(**config), user='odoo'))
+
+    cp_cover_cmd = 'docker cp {container_vol}/. {container_name}:{coverage_reports}/.'.format(**config)
+    subprocess.call(shlex.split(cp_cover_cmd))
+
+    _logger.debug('Changed permissions for coverage %s', exec_cmd(config['container_name'], 'chown -R odoo:odoo {coverage_reports}'.format(**config)))
+
+    exec_cmd(config.get('container_name'), 'pip3 install coverage')
+
+
 def save_logs(config, logs):
     odoo_log = path.join(config.get('container_vol'), 'odoo.log_%(ci_job_name)s' % config)
     _logger.info("Save log in %s cwd %s", odoo_log, getcwd())
@@ -727,3 +746,20 @@ def run_image_tests(config, test_only=False):
         res = test_module(config)
     clean_containers(config)
     return res
+
+
+def generate_coverage_reports(config):
+    clean_containers(config)
+    coverage_env = get_coverage_env_variables(config)
+    config.update({key.lower(): value for key, value in coverage_env.items()})
+    container = _cli.create_container(
+        image=config['instance_image'],
+        name=config['container_name'],
+        environment=coverage_env,
+    )
+    _cli.start(container=container.get('Id'))
+    _logger.info(container)
+    prepare_coverage(config)
+    status, msg = container_coverage_reports(config, combine=True)
+    clean_containers(config)
+    return status, msg
