@@ -1,5 +1,5 @@
 import logging
-from os import environ, path, walk, rename
+from os import environ, path, walk
 import paramiko
 from sys import exit
 
@@ -31,17 +31,13 @@ def get_sftp_client(username, password):
 
 def push_files():
     values = get_values()
-    local_path = path.join('.', '{}-{}'.format(
-        values.get('commit_slug'),
-        values.get('project_name')
-    ))
-    remote_path = '.'
-    _logger.info('Pushing files from: %s', local_path)
-    sftp = get_sftp_client(values.get('username'), values.get('password'))
-    try:
-        rename(values.get('commit_slug'), local_path)
-    except OSError:
-        _logger.warn('Nothing to report from %s', values.get('commit_slug'))
+    # TODO: Match get_config(**kwargs) container_vol value and get_values
+    values.update({'container_vol': values['commit_slug']})
+
+    remote_path = '{commit_slug}-{project_name}'.format(**values)
+    src_path = '{container_vol}/htmlcov'.format(**values)
+    if not path.isdir(src_path):
+        _logger.warn('Nothing to report from %s', src_path)
         _logger.warn('\nThis job is failing because the coverage report folder is empty this is due to:' +
                      '\n  - No tests were executed' +
                      '\n  - The test job is not properly configured' +
@@ -52,9 +48,11 @@ def push_files():
                      '\n https://git.vauxoo.com/vauxoo/project-template/blob/master/%7B%7Bcookiecutter.project_name%7D%7D/.gitlab-ci.yml')
         exit(1)
 
-    for root, dirs, files in walk(local_path):
+    _logger.info('Pushing files from: %s', src_path)
+    sftp = get_sftp_client(values.get('username'), values.get('password'))
+    for root, dirs, files in walk(src_path):
         _logger.debug('Root: %s .. Dirs: %s .. Files: %s', root, dirs, files)
-        remote = path.join(remote_path, root)
+        remote = path.join(path.relpath(root, src_path), remote_path)
         _logger.debug('Creating remote folder: %s', remote)
         try:
             sftp.mkdir(remote)
@@ -63,10 +61,8 @@ def push_files():
             pass
 
         for f in files:
-            _logger.debug('Uploading file: %s', f)
-            if not f.startswith("odoo.log_") or f != '.coveragerc':
-                sftp.put(path.join(root, f), path.join(remote_path, root, f))
-
+            _logger.debug('Uploading file: %s to %s', path.join(root, f), path.join(remote, f))
+            sftp.put(path.join(root, f), path.join(remote, f))
     _logger.info('Pushed files')
 
 
